//+------------------------------------------------------------------+
//|                                               Hurst Exponent.mq4 |
//|                      Copyright  2006, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Copyright  2006, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net"

#property indicator_separate_window
#property indicator_level1 0.45
#property indicator_level2 0.5
#property indicator_level3 0.55
#property indicator_level4 1.0
#property indicator_buffers 4
#property indicator_color1 White
#property indicator_color2 Red
#property indicator_color3 Yellow
#property indicator_color4 Red

#include    <stderror.mqh>
#include    <stdlib.mqh>
//---- input parameters
extern int       cMaxSamples=2520;
extern bool      cRSGraphic=true;
extern bool      logPreprocessing=false;
extern bool      ARPreprocessing=false;
//---- buffers
double B0[];
double B1[];
double B2[];
double B3[];

double d[], x[], y[], v[];
double a, b, h;
int dc, mip, map, xyc;
bool StartUp = true; 

/* computes logarithmic differences */
void ld ()
{
   dc--;
	for (int i=0; i<dc; i++)
     if (d[i] > 0 && d[i+1] > 0) 
      if (d[i+1] / d[i] > 0) 
		 d[i] = MathLog(d[i+1] / d[i]);
}

/* solves linear least squares problem*/
void lls ()
{
	double xs = 0; double ys = 0;	double xxs = 0; double xys =0;
	for (int i = 0; i < xyc; i++)
	{
		xs += x[i];
		ys += y[i];
		xxs += x[i] * x[i];
		xys += x[i] * y[i];
	}
	a = (xs * ys - xyc * xys) / (xs * xs - xyc * xxs);
	b = (xs * xys - ys * xxs) / (xs * xs - xyc * xxs);
	h = a;
}

/* computes autoregressive residuals of first order*/
void ar ()
{
   dc--;
	double xs = 0; double ys = 0;	double xxs = 0; double xys =0;
	for (int i = 0; i < dc; i++)
	{
		xs += d[i];
		ys += d[i+1];
		xxs += d[i] * d[i];
		xys += d[i] * d[i+1];
	}
	a = (xs * ys - xyc * xys) / (xs * xs - xyc * xxs);
	b = (xs * xys - ys * xxs) / (xs * xs - xyc * xxs);
	for (i = 0; i < dc; i++) d[i] = d[i+1] - (a * d[i] + b);
}

/* computes a number of divisors */
int dn(int x, int mid, int mad)
{
	int i, n = 0; 
	for (i = mid; i <= mad; i++) if (x % i == 0) n++;
	return (n);
}

/* generates selection */
bool gs()
{
	int i, j, n, max = 0;

	mip = 10;
	map = dc / 2;
	j = 0.9 * dc;
	for (i = j; i <= dc; i++)
	{
		n = dn(i, mip, MathMin(i,map));
		if (max <= n) { max = n; j = i; }
	}
	if (max < 2) { Print ("Not enough data"); return (false); }

//	for (p = &d[*dc - j], de = &d[j]; d < de; ++p, ++d) *d = *p;

	dc = j;
	return (true);
}

/* executes r/s analysis */
void rs()
{
	int i, j, k, n;
	double mean, max, min, sum, rs;
	for (i = mip, xyc =0; i <= map; i++)
		if (dc % i == 0)
		{
			n = dc / i;
			for (j = 0, rs = 0; j < n; j++)	
			{
				for (k = 0, mean = 0; k < i; k++)
					mean += d[j * i + k];
				mean /= i;
				max = d[j * i] - mean;
				min = max;
				for (k = 0, sum = 0; k < i; k++)
				{
					sum += d[j * i + k] - mean;
					if (sum < min) min = sum;
					if (sum > max) max = sum;
				}
				for (k = 0, sum = 0; k < i; k++)
					sum += (d[j * i + k] - mean) * (d[j * i + k] - mean);
				sum = MathSqrt(sum / i);
				if (sum == 0) { rs = 0; break; }
				rs += (max - min) / sum;
			}
			rs /= n;
			if (rs != 0)
			{
				x[xyc] = MathLog(i);
				y[xyc] = MathLog(rs);
				v[xyc] = rs / MathSqrt(i); 
				xyc++;
			}
			//else Print("rs = 0!");
		}
}

/* computes hurst exponent */
double hurst ()
{
	if (logPreprocessing) ld();              /* !optional! logarithmic preprocessing */
	if (ARPreprocessing) ar();               /* !optional! autoregression preprocessing */
	if (gs())                                /* selection generation */
	  {
	  rs();                                  /* r/s analysis */
	  lls();                                 /* linear least squares solution*/
	  }
}

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   IndicatorBuffers(4);
//---- indicator buffers
   SetIndexBuffer(0,B0);
   SetIndexBuffer(1,B1);
   SetIndexBuffer(2,B2);
   SetIndexBuffer(3,B3);
//---- drawing parameters setting
    SetIndexStyle(0,DRAW_LINE);
    SetIndexStyle(1,DRAW_LINE);
    SetIndexStyle(2,DRAW_LINE);
    SetIndexStyle(3,DRAW_LINE);
//---- name for DataWindow and indicator subwindow label
   IndicatorShortName("Hurst Exponent");
//----
//   ArrayInitialize(B0,0);
//   ArrayInitialize(B1,0);
//   ArrayInitialize(B2,0);
   ArrayResize(d, MathMax(1024, cMaxSamples));
   ArrayResize(x, MathMax(1024, cMaxSamples));
   ArrayResize(y, MathMax(1024, cMaxSamples));
   ArrayResize(v, MathMax(1024, cMaxSamples));

   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int TestDataCheck()
   {
//----
   int handle;
   handle=FileOpen("brown72.csv",FILE_CSV|FILE_READ,';');
   if(handle<1)
   {
      Print("File brown72.csv is not opened, the last error is ", ErrorDescription(GetLastError()));
      return(false);
   }
   for (int i=0;i<1024;i++) d[i]=FileReadNumber(handle); 
   FileClose(handle);
   dc=1024;
   hurst();
   Print("Test Hurst exponent: ", h);
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
   int limit;
   int counted_bars=IndicatorCounted();
//---- check for possible errors
   if(counted_bars<0) return(-1);
//---- last counted bar will be recounted
   if(counted_bars>0) counted_bars--;
   limit=Bars-counted_bars;
//---- main loop
   for(int i=0; i<limit; i++)
     {
     }
     
//---- done
   if (StartUp) TestDataCheck();

   if (cRSGraphic)
      {
      if (Bars <= cMaxSamples) { Print ("Not enough bars"); return (false); }
      for(i=0, dc = cMaxSamples; i<dc; i++) d[i] = 2*i + 1;
//      for(i=0, dc = cMaxSamples; i<dc; i++) d[i] = Close[i + 1] - Close[i];
      hurst();				               /* computation of hurst exponent */
      Print("Hurst exponent: ", h);
      Comment("H = ",DoubleToStr(h,4),"   Data count: ", dc, " Number of Selections: ", xyc);
		
      double maxRS=y[ArrayMaximum(y,xyc,0)]/xyc;
      double minRS=y[ArrayMinimum(y,xyc,0)]/xyc;
      for(i = 0; i < xyc; i++) { B0[i] = y[i]; B1[i] = (maxRS - minRS) * i; B2[i]=v[i]; }
      StartUp = false; 
      return(0);
      }
   if (StartUp) limit = 250;
   for(int cnt = 0; cnt < limit; cnt++)
      {
      if (Bars <= cMaxSamples + cnt) { Print ("Not enough bars"); return (false); }
      for(i = 0, dc = cMaxSamples; i < dc; i++) d[i] = Close[cnt + i + 1] - Close[cnt + i];
      hurst();				               /* computation of hurst exponent */
      B0[cnt]=h;
      }
   Comment("H = ",DoubleToStr(B0[0],4),"   Data count: ", dc, " Number of Selections: ", xyc);
   StartUp = false;   
   return(0);
  }
//+------------------------------------------------------------------+

